@@ -19,12 +19,7 @@
 ** or binary data.
 */
 #include "config.h"
-#if defined(FOSSIL_ENABLE_MINIZ)
-#  define MINIZ_HEADER_FILE_ONLY
-#  include "miniz.c"
-#else
-#  include <zlib.h>
-#endif
+#include <zlib.h>
 #include "blob.h"
 #if defined(_WIN32)
 #include <fcntl.h>
@@ -60,6 +55,29 @@ struct Blob {
 */
 #define blob_buffer(X)  ((X)->aData)
 
+/*
+** Append blob contents to another
+*/
+#define blob_appendb(dest, src) \
+  blob_append((dest), blob_buffer(src), blob_size(src))
+
+/*
+** Append a string literal to a blob
+** TODO: Consider renaming to blob_appendl()
+*/
+#define blob_append_literal(blob, literal) \
+  blob_append((blob), "" literal, (sizeof literal)-1)
+  /*
+   * The empty string in the second argument leads to a syntax error
+   * when the macro is not used with a string literal. Unfortunately
+   * the error is not overly explicit.
+   */
+
+/*
+** TODO: Suggested for removal because the name seems misleading.
+*/
+#define blob_append_string blob_append_literal
+
 /*
 ** Seek whence parameter values
 */
@@ -171,7 +189,7 @@ void blobReallocMalloc(Blob *pBlob, unsigned int newSize){
     pBlob->nUsed = 0;
     pBlob->iCursor = 0;
     pBlob->blobFlags = 0;
-  }else if( newSize>pBlob->nAlloc || newSize<pBlob->nAlloc-4000 ){
+  }else if( newSize>pBlob->nAlloc || newSize+4000<pBlob->nAlloc ){
     char *pNew = fossil_realloc(pBlob->aData, newSize);
     pBlob->aData = pNew;
     pBlob->nAlloc = newSize;
@@ -332,13 +350,6 @@ void blob_append(Blob *pBlob, const char *aData, int nData){
   memcpy(&pBlob->aData[nUsed], aData, nData);
 }
 
-/*
-** Append a string literal to a blob.
-*/
-#if INTERFACE
-#define blob_append_string(BLOB,STR) blob_append(BLOB,STR,sizeof(STR)-1)
-#endif
-
 /*
 ** Append a single character to the blob.  If pBlob is zero then the
 ** character is written directly to stdout.
@@ -510,7 +521,7 @@ char *blob_terminate(Blob *p){
 ** Compare two blobs.  Return negative, zero, or positive if the first
 ** blob is less then, equal to, or greater than the second.
 */
-int blob_compare(Blob *pA, Blob *pB){
+int blob_compare(const Blob *pA, const Blob *pB){
   int szA, szB, sz, rc;
   blob_is_init(pA);
   blob_is_init(pB);
@@ -602,8 +613,8 @@ void blob_resize(Blob *pBlob, unsigned int newSize){
 void blob_reserve(Blob *pBlob, unsigned int newSize){
   if(newSize>=0x7fff0000 ){
     blob_panic();
-  }else if(newSize>pBlob->nUsed){
-    pBlob->xRealloc(pBlob, newSize);
+  }else if(newSize>pBlob->nAlloc){
+    pBlob->xRealloc(pBlob, newSize+1);
     pBlob->aData[newSize] = 0;
   }
 }
@@ -978,6 +989,30 @@ int blob_read_from_channel(Blob *pBlob, FILE *in, int nToRead){
   return blob_size(pBlob);
 }
 
+/*
+** Initialize a blob to the data read from HTTP input.  Return
+** the number of bytes read into the blob.  Any prior content
+** of the blob is discarded, not freed.
+*/
+int blob_read_from_cgi(Blob *pBlob, int nToRead){
+  size_t n;
+  blob_zero(pBlob);
+  if( nToRead<0 ){
+    char zBuf[10000];
+    while( !cgi_feof() ){
+      n = cgi_fread(zBuf, sizeof(zBuf));
+      if( n>0 ){
+        blob_append(pBlob, zBuf, n);
+      }
+    }
+  }else{
+    blob_resize(pBlob, nToRead);
+    n = cgi_fread(blob_buffer(pBlob), nToRead);
+    blob_resize(pBlob, n);
+  }
+  return blob_size(pBlob);
+}
+
 /*
 ** Initialize a blob to be the content of a file.  If the filename
 ** is blank or "-" then read from standard input.
