@@ -276,6 +276,7 @@ void test_show_vfile_cmd(void){
 
 /*
 ** COMMAND: merge
+** COMMAND: cherry-pick
 **
 ** Usage: %fossil merge ?OPTIONS? ?VERSION?
 **
@@ -285,7 +286,8 @@ void test_show_vfile_cmd(void){
 ** --backout options are used only the changes associated with the
 ** single check-in VERSION are merged.  The --backout option causes
 ** the changes associated with VERSION to be removed from the current
-** checkout rather than added.
+** checkout rather than added. When invoked with the name cherry-pick,
+** this command works exactly like merge --cherrypick.
 **
 ** If the VERSION argument is omitted, then Fossil attempts to find
 ** a recent fork on the current branch to merge.
@@ -309,10 +311,6 @@ void test_show_vfile_cmd(void){
 **                           and do not try to merge parallel changes.  This
 **                           option overrides the "binary-glob" setting.
 **
-**   --case-sensitive BOOL   Override the case-sensitive setting.  If false,
-**                           files whose names differ only in case are taken
-**                           to be the same file.
-**
 **   --cherrypick            Do a cherrypick merge VERSION into the current
 **                           checkout.  A cherrypick merge pulls in the changes
 **                           of the single check-in VERSION, rather than all
@@ -370,6 +368,9 @@ void merge_cmd(void){
     verboseFlag = find_option("detail",0,0)!=0; /* deprecated */
   }
   pickFlag = find_option("cherrypick",0,0)!=0;
+  if('c'==*g.zCmdName/*called as cherry-pick, possibly a short form*/){
+    pickFlag = 1;
+  }
   integrateFlag = find_option("integrate",0,0)!=0;
   backoutFlag = find_option("backout",0,0)!=0;
   zBinGlob = find_option("binary",0,1);
@@ -408,8 +409,7 @@ void merge_cmd(void){
     fossil_fatal("cannot merge into a closed leaf. Use --force to override");
   }
   if( !dryRunFlag ){
-    if( autosync_loop(SYNC_PULL + SYNC_VERBOSE*verboseFlag,
-                      db_get_int("autosync-tries", 1), 1) ){
+    if( autosync_loop(SYNC_PULL + SYNC_VERBOSE*verboseFlag, 1, "merge") ){
       fossil_fatal("merge abandoned due to sync failure");
     }
   }
